import {
  ExceptionFilter,
  Catch,
  ArgumentsHost,
  HttpException,
} from '@nestjs/common';
import { ApolloError } from 'apollo-server-errors';

@Catch(HttpException)
export class HttpExceptionFilter implements ExceptionFilter {
  catch(exception: HttpException, host: ArgumentsHost) {
    const message = exception.message;
    const extensions: Record<string, any> = {};
    const err = exception.getResponse();
    if (typeof err === 'string') {
      extensions.message = err;
    } else {
      Object.assign(extensions, (err as any).extension);
      extensions.message = (err as any).message;
    }
    return new ApolloError(
      message,
      exception.getStatus().toString(),
      extensions,
    );
  }
}
